=== modified file 'drizzled/cursor.cc'
--- drizzled/cursor.cc	2010-08-28 01:11:17 +0000
+++ drizzled/cursor.cc	2010-09-09 18:27:05 +0000
@@ -95,7 +95,6 @@
 
   if (new_handler && !new_handler->ha_open(identifier,
                                            table,
-                                           table->getMutableShare()->getNormalizedPath(),
                                            table->getDBStat(),
                                            HA_OPEN_IGNORE_IF_LOCKED))
     return new_handler;
@@ -229,7 +228,8 @@
   Don't wait for locks if not HA_OPEN_WAIT_IF_LOCKED is set
 */
 int Cursor::ha_open(const TableIdentifier &identifier,
-                    Table *table_arg, const char *name, int mode,
+                    Table *table_arg,
+                    int mode,
                     int test_if_locked)
 {
   int error;
@@ -237,7 +237,6 @@
   table= table_arg;
   assert(table->getShare() == table_share);
 
-  assert(identifier.getPath().compare(name) == 0);
   if ((error= doOpen(identifier, mode, test_if_locked)))
   {
     if ((error == EACCES || error == EROFS) && mode == O_RDWR &&

=== modified file 'drizzled/cursor.h'
--- drizzled/cursor.h	2010-08-17 01:34:55 +0000
+++ drizzled/cursor.h	2010-09-09 18:27:05 +0000
@@ -229,7 +229,7 @@
 
   /* ha_ methods: pubilc wrappers for private virtual API */
 
-  int ha_open(const TableIdentifier &identifier, Table *table, const char *name, int mode, int test_if_locked);
+  int ha_open(const TableIdentifier &identifier, Table *table, int mode, int test_if_locked);
   int startIndexScan(uint32_t idx, bool sorted);
   int endIndexScan();
   int startTableScan(bool scan);

=== modified file 'drizzled/include.am'
--- drizzled/include.am	2010-08-20 18:39:24 +0000
+++ drizzled/include.am	2010-09-09 21:45:53 +0000
@@ -777,7 +777,7 @@
 				 drizzled/transaction_services.cc \
 				 drizzled/tree.cc \
 				 drizzled/tztime.cc \
-				 drizzled/uniques.cc \
+				 drizzled/unique.cc \
 				 drizzled/user_var_entry.cc \
 				 drizzled/version.cc \
 				 drizzled/xid.cc

=== modified file 'drizzled/lock.cc'
--- drizzled/lock.cc	2010-08-17 01:34:55 +0000
+++ drizzled/lock.cc	2010-09-10 03:56:45 +0000
@@ -569,13 +569,13 @@
 static DrizzleLock *get_lock_data(Session *session, Table **table_ptr, uint32_t count,
 				 bool should_lock, Table **write_lock_used)
 {
-  uint32_t i,tables,lock_count;
+  uint32_t tables,lock_count;
   DrizzleLock *sql_lock;
   THR_LOCK_DATA **locks, **locks_buf, **locks_start;
   Table **to, **table_buf;
 
   *write_lock_used=0;
-  for (i= tables= lock_count= 0 ; i < count ; i++)
+  for (uint32_t i= tables= lock_count= 0 ; i < count ; i++)
   {
     Table *t= table_ptr[i];
 
@@ -596,12 +596,14 @@
 	malloc(sizeof(*sql_lock) +
                sizeof(THR_LOCK_DATA*) * tables * 2 +
                sizeof(table_ptr) * lock_count)))
+  {
     return NULL;
+  }
   locks= locks_buf= sql_lock->locks= (THR_LOCK_DATA**) (sql_lock + 1);
   to= table_buf= sql_lock->table= (Table**) (locks + tables * 2);
   sql_lock->table_count= lock_count;
 
-  for (i=0 ; i < count ; i++)
+  for (uint32_t i= 0; i < count ; i++)
   {
     Table *table;
     enum thr_lock_type lock_type;

=== modified file 'drizzled/records.cc'
--- drizzled/records.cc	2010-08-18 00:30:07 +0000
+++ drizzled/records.cc	2010-09-09 23:36:35 +0000
@@ -420,7 +420,6 @@
 
 static int rr_from_cache(ReadRecord *info)
 {
-  register uint32_t i;
   uint32_t length;
   internal::my_off_t rest_of_file;
   int16_t error;
@@ -460,7 +459,7 @@
     length/=info->ref_length;
     position=info->getCache();
     ref_position=info->read_positions;
-    for (i=0 ; i < length ; i++,position+=info->ref_length)
+    for (uint32_t i= 0 ; i < length ; i++,position+=info->ref_length)
     {
       memcpy(ref_position,position,(size_t) info->ref_length);
       ref_position+=MAX_REFLENGTH;
@@ -471,7 +470,7 @@
                        (qsort_cmp) rr_cmp);
 
     position=info->read_positions;
-    for (i=0 ; i < length ; i++)
+    for (uint32_t i= 0 ; i < length ; i++)
     {
       memcpy(info->ref_pos, position, (size_t)info->ref_length);
       position+=MAX_REFLENGTH;

=== modified file 'drizzled/sql_base.cc'
--- drizzled/sql_base.cc	2010-08-18 23:12:07 +0000
+++ drizzled/sql_base.cc	2010-09-09 18:39:55 +0000
@@ -47,7 +47,6 @@
 #include "drizzled/cached_directory.h"
 #include <drizzled/field/timestamp.h>
 #include <drizzled/field/null.h>
-#include "drizzled/memory/multi_malloc.h"
 #include "drizzled/sql_table.h"
 #include "drizzled/global_charset_info.h"
 #include "drizzled/pthread_globals.h"
@@ -1081,8 +1080,6 @@
 
   /*
     Create a table entry with the right key and with an old refresh version
-    Note that we must use multi_malloc() here as this is freed by the
-    table cache
   */
   TableIdentifier identifier(db_name, table_name, message::Table::INTERNAL);
   TablePlaceholder *table= new TablePlaceholder(this, identifier);

=== modified file 'drizzled/sql_insert.cc'
--- drizzled/sql_insert.cc	2010-08-17 01:34:55 +0000
+++ drizzled/sql_insert.cc	2010-09-09 18:39:55 +0000
@@ -716,7 +716,7 @@
 int write_record(Session *session, Table *table,CopyInfo *info)
 {
   int error;
-  char *key=0;
+  std::vector<unsigned char> key;
   MyBitmap *save_read_set, *save_write_set;
   uint64_t prev_insert_id= table->cursor->next_insert_id;
   uint64_t insert_id_for_cur_row= 0;
@@ -787,17 +787,13 @@
 	  goto err;
 	}
 
-	if (!key)
+	if (not key.size())
 	{
-	  if (!(key=(char*) malloc(table->getShare()->max_unique_length)))
-	  {
-	    error=ENOMEM;
-	    goto err;
-	  }
+          key.resize(table->getShare()->max_unique_length);
 	}
-	key_copy((unsigned char*) key,table->getInsertRecord(),table->key_info+key_nr,0);
+	key_copy(&key[0], table->getInsertRecord(), table->key_info+key_nr, 0);
 	if ((error=(table->cursor->index_read_idx_map(table->getUpdateRecord(),key_nr,
-                                                    (unsigned char*) key, HA_WHOLE_KEY,
+                                                    &key[0], HA_WHOLE_KEY,
                                                     HA_READ_KEY_EXACT))))
 	  goto err;
       }
@@ -931,8 +927,6 @@
   session->record_first_successful_insert_id_in_cur_stmt(table->cursor->insert_id_for_cur_row);
 
 gok_or_after_err:
-  if (key)
-    free(key);
   if (!table->cursor->has_transactions())
     session->transaction.stmt.markModifiedNonTransData();
   return(0);
@@ -946,8 +940,6 @@
 
 before_err:
   table->cursor->restore_auto_increment(prev_insert_id);
-  if (key)
-    free(key);
   table->column_bitmaps_set(save_read_set, save_write_set);
   return(1);
 }

=== modified file 'drizzled/table.cc'
--- drizzled/table.cc	2010-08-18 03:47:58 +0000
+++ drizzled/table.cc	2010-09-09 18:27:05 +0000
@@ -1565,7 +1565,6 @@
   TableIdentifier identifier(s->getSchemaName(), s->getTableName(), s->getPath());
   if ((error=cursor->ha_open(identifier,
                              this,
-                             s->getTableName(),
                              O_RDWR,
                              HA_OPEN_TMP_TABLE | HA_OPEN_INTERNAL_TABLE)))
   {

=== modified file 'drizzled/table_share.cc'
--- drizzled/table_share.cc	2010-08-07 17:38:08 +0000
+++ drizzled/table_share.cc	2010-09-09 18:27:05 +0000
@@ -1978,7 +1978,7 @@
     assert(!(db_stat & HA_WAIT_IF_LOCKED));
     int ha_err;
 
-    if ((ha_err= (outparam.cursor->ha_open(identifier, &outparam, getNormalizedPath(),
+    if ((ha_err= (outparam.cursor->ha_open(identifier, &outparam,
                           (db_stat & HA_READ_ONLY ? O_RDONLY : O_RDWR),
                           (db_stat & HA_OPEN_TEMPORARY ? HA_OPEN_TMP_TABLE : HA_OPEN_IGNORE_IF_LOCKED) | ha_open_flags))))
     {

=== renamed file 'drizzled/uniques.cc' => 'drizzled/unique.cc'
--- drizzled/uniques.cc	2010-08-17 01:34:55 +0000
+++ drizzled/unique.cc	2010-09-09 21:45:53 +0000
@@ -21,10 +21,6 @@
   The basic idea is as follows:
 
   Store first all strings in a binary tree, ignoring duplicates.
-  When the tree uses more memory than 'max_heap_table_size',
-  write the tree (in sorted order) out to disk and start with a new tree.
-  When all data has been generated, merge the trees (removing any found
-  duplicates).
 
   The unique entries will be returned in sort order, to ensure that we do the
   deletes in disk order.
@@ -50,18 +46,6 @@
 namespace drizzled
 {
 
-int unique_write_to_file(unsigned char* key, uint32_t,
-                         Unique *unique)
-{
-  /*
-    Use unique->size (size of element stored in the tree) and not
-    unique->tree.size_of_element. The latter is different from unique->size
-    when tree implementation chooses to store pointer to key in TREE_ELEMENT
-    (instead of storing the element itself there)
-  */
-  return my_b_write(unique->file, key, unique->size) ? 1 : 0;
-}
-
 int unique_write_to_ptrs(unsigned char* key,
                          uint32_t, Unique *unique)
 {
@@ -73,22 +57,12 @@
 Unique::Unique(qsort_cmp2 comp_func, void * comp_func_fixed_arg,
 	       uint32_t size_arg, size_t max_in_memory_size_arg)
   : max_in_memory_size(max_in_memory_size_arg),
-    file(static_cast<internal::IO_CACHE *>(memory::sql_calloc(sizeof(internal::IO_CACHE)))),
     size(size_arg),
     elements(0)
 {
-  my_b_clear(file);
-  init_tree(&tree, (ulong) (max_in_memory_size / 16), 0, size, comp_func, false,
+  // Second element is max size for memory use in unique sort
+  init_tree(&tree, 0, 0, size, comp_func, false,
             NULL, comp_func_fixed_arg);
-  /* If the following fail's the next add will also fail */
-  my_init_dynamic_array(&file_ptrs, sizeof(buffpek_st), 16, 16);
-  /*
-    If you change the following, change it in get_max_elements function, too.
-  */
-  max_elements= (ulong) (max_in_memory_size /
-                         ALIGN_SIZE(sizeof(TREE_ELEMENT)+size));
-  open_cached_file(file, drizzle_tmpdir.c_str(), TEMP_PREFIX, DISK_BUFFER_SIZE,
-                   MYF(MY_WME));
 }
 
 
@@ -114,125 +88,6 @@
 
 
 /*
-  Calculate cost of merge_buffers function call for given sequence of
-  input stream lengths and store the number of rows in result stream in *last.
-
-  SYNOPSIS
-    get_merge_buffers_cost()
-      buff_elems  Array of #s of elements in buffers
-      elem_size   Size of element stored in buffer
-      first       Pointer to first merged element size
-      last        Pointer to last merged element size
-
-  RETURN
-    Cost of merge_buffers operation in disk seeks.
-
-  NOTES
-    It is assumed that no rows are eliminated during merge.
-    The cost is calculated as
-
-      cost(read_and_write) + cost(merge_comparisons).
-
-    All bytes in the sequences is read and written back during merge so cost
-    of disk io is 2*elem_size*total_buf_elems/IO_SIZE (2 is for read + write)
-
-    For comparisons cost calculations we assume that all merged sequences have
-    the same length, so each of total_buf_size elements will be added to a sort
-    heap with (n_buffers-1) elements. This gives the comparison cost:
-
-      total_buf_elems* log2(n_buffers) / TIME_FOR_COMPARE_ROWID;
-*/
-
-static double get_merge_buffers_cost(uint32_t *, uint32_t elem_size,
-                                     uint32_t *first, uint32_t *last)
-{
-  uint32_t total_buf_elems= 0;
-  for (uint32_t *pbuf= first; pbuf <= last; pbuf++)
-    total_buf_elems+= *pbuf;
-  *last= total_buf_elems;
-
-  int n_buffers= last - first + 1;
-
-  /* Using log2(n)=log(n)/log(2) formula */
-  return 2*((double)total_buf_elems*elem_size) / IO_SIZE +
-     total_buf_elems*log((double) n_buffers) / (TIME_FOR_COMPARE_ROWID * M_LN2);
-}
-
-
-/*
-  Calculate cost of merging buffers into one in Unique::get, i.e. calculate
-  how long (in terms of disk seeks) the two calls
-    merge_many_buffs(...);
-    merge_buffers(...);
-  will take.
-
-  SYNOPSIS
-    get_merge_many_buffs_cost()
-      buffer        buffer space for temporary data, at least
-                    Unique::get_cost_calc_buff_size bytes
-      maxbuffer     # of full buffers
-      max_n_elems   # of elements in first maxbuffer buffers
-      last_n_elems  # of elements in last buffer
-      elem_size     size of buffer element
-
-  NOTES
-    maxbuffer+1 buffers are merged, where first maxbuffer buffers contain
-    max_n_elems elements each and last buffer contains last_n_elems elements.
-
-    The current implementation does a dumb simulation of merge_many_buffs
-    function actions.
-
-  RETURN
-    Cost of merge in disk seeks.
-*/
-
-static double get_merge_many_buffs_cost(uint32_t *buffer,
-                                        uint32_t maxbuffer, uint32_t max_n_elems,
-                                        uint32_t last_n_elems, int elem_size)
-{
-  register int i;
-  double total_cost= 0.0;
-  uint32_t *buff_elems= buffer; /* #s of elements in each of merged sequences */
-
-  /*
-    Set initial state: first maxbuffer sequences contain max_n_elems elements
-    each, last sequence contains last_n_elems elements.
-  */
-  for (i = 0; i < (int)maxbuffer; i++)
-    buff_elems[i]= max_n_elems;
-  buff_elems[maxbuffer]= last_n_elems;
-
-  /*
-    Do it exactly as merge_many_buff function does, calling
-    get_merge_buffers_cost to get cost of merge_buffers.
-  */
-  if (maxbuffer >= MERGEBUFF2)
-  {
-    while (maxbuffer >= MERGEBUFF2)
-    {
-      uint32_t lastbuff= 0;
-      for (i = 0; i <= (int) maxbuffer - MERGEBUFF*3/2; i += MERGEBUFF)
-      {
-        total_cost+=get_merge_buffers_cost(buff_elems, elem_size,
-                                           buff_elems + i,
-                                           buff_elems + i + MERGEBUFF-1);
-	lastbuff++;
-      }
-      total_cost+=get_merge_buffers_cost(buff_elems, elem_size,
-                                         buff_elems + i,
-                                         buff_elems + maxbuffer);
-      maxbuffer= lastbuff;
-    }
-  }
-
-  /* Simulate final merge_buff call. */
-  total_cost += get_merge_buffers_cost(buff_elems, elem_size,
-                                       buff_elems, buff_elems + maxbuffer);
-  return total_cost;
-}
-
-
-/*
   Calculate cost of using Unique for processing nkeys elements of size
   key_size using max_in_memory_size memory.
 
@@ -267,6 +122,9 @@
 
       Approximate value of log2(N!) is calculated by log2_n_fact function.
 
+       
+      (The Next two are historical, we do all unique operations in memory or fail)
+
     2. Cost of merging.
       If only one tree is created by Unique no merging will be necessary.
       Otherwise, we model execution of merge_many_buff function and count
@@ -279,83 +137,33 @@
       these will be random seeks.
 */
 
-double Unique::get_use_cost(uint32_t *buffer, uint32_t nkeys, uint32_t key_size,
+double Unique::get_use_cost(uint32_t *, uint32_t nkeys, uint32_t key_size,
                             size_t max_in_memory_size)
 {
   ulong max_elements_in_tree;
   ulong last_tree_elems;
-  int   n_full_trees; /* number of trees in unique - 1 */
   double result;
 
   max_elements_in_tree= ((ulong) max_in_memory_size /
                          ALIGN_SIZE(sizeof(TREE_ELEMENT)+key_size));
 
-  n_full_trees=    nkeys / max_elements_in_tree;
   last_tree_elems= nkeys % max_elements_in_tree;
 
   /* Calculate cost of creating trees */
   result= 2*log2_n_fact(last_tree_elems + 1.0);
-  if (n_full_trees)
-    result+= n_full_trees * log2_n_fact(max_elements_in_tree + 1.0);
   result /= TIME_FOR_COMPARE_ROWID;
 
-
-  if (!n_full_trees)
-    return result;
-
-  /*
-    There is more then one tree and merging is necessary.
-    First, add cost of writing all trees to disk, assuming that all disk
-    writes are sequential.
-  */
-  result += DISK_SEEK_BASE_COST * n_full_trees *
-              ceil(((double) key_size)*max_elements_in_tree / IO_SIZE);
-  result += DISK_SEEK_BASE_COST * ceil(((double) key_size)*last_tree_elems / IO_SIZE);
-
-  /* Cost of merge */
-  double merge_cost= get_merge_many_buffs_cost(buffer, n_full_trees,
-                                               max_elements_in_tree,
-                                               last_tree_elems, key_size);
-  if (merge_cost < 0.0)
-    return merge_cost;
-
-  result += merge_cost;
-  /*
-    Add cost of reading the resulting sequence, assuming there were no
-    duplicate elements.
-  */
-  result += ceil((double)key_size*nkeys/IO_SIZE);
-
   return result;
 }
 
 Unique::~Unique()
 {
-  close_cached_file(file);
-  delete_tree(&tree);
-  delete_dynamic(&file_ptrs);
-}
-
-
-    /* Write tree to disk; clear tree */
-bool Unique::flush()
-{
-  buffpek_st file_ptr;
-  elements+= tree.elements_in_tree;
-  file_ptr.count=tree.elements_in_tree;
-  file_ptr.file_pos=my_b_tell(file);
-
-  if (tree_walk(&tree, (tree_walk_action) unique_write_to_file,
-		(void*) this, left_root_right) ||
-      insert_dynamic(&file_ptrs, (unsigned char*) &file_ptr))
-    return 1;
-  delete_tree(&tree);
-  return 0;
+  delete_tree(&tree);
 }
 
 
 /*
-  Clear the tree and the file.
+  Clear the tree.
   You must call reset() if you want to reuse Unique after walk().
 */
 
@@ -363,196 +171,7 @@
 Unique::reset()
 {
   reset_tree(&tree);
-  /*
-    If elements != 0, some trees were stored in the file (see how
-    flush() works). Note, that we can not count on my_b_tell(&file) == 0
-    here, because it can return 0 right after walk(), and walk() does not
-    reset any Unique member.
-  */
-  if (elements)
-  {
-    reset_dynamic(&file_ptrs);
-    reinit_io_cache(file, internal::WRITE_CACHE, 0L, 0, 1);
-  }
-  elements= 0;
-}
-
-/*
-  The comparison function, passed to queue_init() in merge_walk() and in
-  merge_buffers() when the latter is called from Uniques::get() must
-  use comparison function of Uniques::tree, but compare members of struct
-  buffpek_st.
-*/
-
-static int buffpek_compare(void *arg, unsigned char *key_ptr1, unsigned char *key_ptr2)
-{
-  BUFFPEK_COMPARE_CONTEXT *ctx= (BUFFPEK_COMPARE_CONTEXT *) arg;
-  return ctx->key_compare(ctx->key_compare_arg,
-                          *((unsigned char **) key_ptr1), *((unsigned char **)key_ptr2));
-}
-
-/*
- The comparison function object, passed to a priority_queue in merge_walk()
- as its sort function parameter.
-*/
-
-class buffpek_compare_functor
-{
-  qsort_cmp2 key_compare;
-  void *key_compare_arg;
-  public:
-  buffpek_compare_functor(qsort_cmp2 in_key_compare, void *in_compare_arg)
-    : key_compare(in_key_compare), key_compare_arg(in_compare_arg) { }
-  inline bool operator()(const buffpek_st *i, const buffpek_st *j)
-  {
-    return key_compare(key_compare_arg,
-                    i->key, j->key);
-  }
-};
-
-/*
-  DESCRIPTION
-
-    Function is very similar to merge_buffers, but instead of writing sorted
-    unique keys to the output file, it invokes walk_action for each key.
-    This saves I/O if you need to pass through all unique keys only once.
-
-  SYNOPSIS
-    merge_walk()
-  All params are 'IN' (but see comment for begin, end):
-    merge_buffer       buffer to perform cached piece-by-piece loading
-                       of trees; initially the buffer is empty
-    merge_buffer_size  size of merge_buffer. Must be aligned with
-                       key_length
-    key_length         size of tree element; key_length * (end - begin)
-                       must be less or equal than merge_buffer_size.
-    begin              pointer to buffpek_st struct for the first tree.
-    end                pointer to buffpek_st struct for the last tree;
-                       end > begin and [begin, end) form a consecutive
-                       range. buffpek_sts structs in that range are used and
-                       overwritten in merge_walk().
-    walk_action        element visitor. Action is called for each unique
-                       key.
-    walk_action_arg    argument to walk action. Passed to it on each call.
-    compare            elements comparison function
-    compare_arg        comparison function argument
-    file               file with all trees dumped. Trees in the file
-                       must contain sorted unique values. Cache must be
-                       initialized in read mode.
-  RETURN VALUE
-    0     ok
-    <> 0  error
-*/
-
-static bool merge_walk(unsigned char *merge_buffer, ulong merge_buffer_size,
-                       uint32_t key_length, buffpek_st *begin, buffpek_st *end,
-                       tree_walk_action walk_action, void *walk_action_arg,
-                       qsort_cmp2 compare, void *compare_arg,
-                       internal::IO_CACHE *file)
-{
-  if (end <= begin ||
-      merge_buffer_size < (ulong) (key_length * (end - begin + 1))) 
-    return 1;
-  priority_queue<buffpek_st *, vector<buffpek_st *>, buffpek_compare_functor >
-    queue(buffpek_compare_functor(compare, compare_arg));
-  /* we need space for one key when a piece of merge buffer is re-read */
-  merge_buffer_size-= key_length;
-  unsigned char *save_key_buff= merge_buffer + merge_buffer_size;
-  uint32_t max_key_count_per_piece= (uint32_t) (merge_buffer_size/(end-begin) /
-                                        key_length);
-  /* if piece_size is aligned reuse_freed_buffer will always hit */
-  uint32_t piece_size= max_key_count_per_piece * key_length;
-  uint32_t bytes_read;               /* to hold return value of read_to_buffer */
-  buffpek_st *top;
-  int res= 1;
-  /*
-    Invariant: queue must contain top element from each tree, until a tree
-    is not completely walked through.
-    Here we're forcing the invariant, inserting one element from each tree
-    to the queue.
-  */
-  for (top= begin; top != end; ++top)
-  {
-    top->base= merge_buffer + (top - begin) * piece_size;
-    top->max_keys= max_key_count_per_piece;
-    bytes_read= read_to_buffer(file, top, key_length);
-    if (bytes_read == (uint32_t) (-1))
-      goto end;
-    assert(bytes_read);
-    queue.push(top);
-  }
-  top= queue.top();
-  while (queue.size() > 1)
-  {
-    /*
-      Every iteration one element is removed from the queue, and one is
-      inserted by the rules of the invariant. If two adjacent elements on
-      the top of the queue are not equal, biggest one is unique, because all
-      elements in each tree are unique. Action is applied only to unique
-      elements.
-    */
-    void *old_key= top->key;
-    /*
-      read next key from the cache or from the file and push it to the
-      queue; this gives new top.
-    */
-    top->key+= key_length;
-    if (--top->mem_count)
-    {
-      queue.pop();
-      queue.push(top);
-    }
-    else /* next piece should be read */
-    {
-      /* save old_key not to overwrite it in read_to_buffer */
-      memcpy(save_key_buff, old_key, key_length);
-      old_key= save_key_buff;
-      bytes_read= read_to_buffer(file, top, key_length);
-      if (bytes_read == (uint32_t) (-1))
-        goto end;
-      else if (bytes_read > 0) /* top->key, top->mem_count are reset */
-      {                        /* in read_to_buffer */
-        queue.pop();
-        queue.push(top);
-      }
-      else
-      {
-        /*
-          Tree for old 'top' element is empty: remove it from the queue. 
-        */
-        queue.pop();
-      }
-    }
-    top= queue.top();
-    /* new top has been obtained; if old top is unique, apply the action */
-    if (compare(compare_arg, old_key, top->key))
-    {
-      if (walk_action(old_key, 1, walk_action_arg))
-        goto end;
-    }
-  }
-  /*
-    Applying walk_action to the tail of the last tree: this is safe because
-    either we had only one tree in the beginning, either we work with the
-    last tree in the queue.
-  */
-  do
-  {
-    do
-    {
-      if (walk_action(top->key, 1, walk_action_arg))
-        goto end;
-      top->key+= key_length;
-    }
-    while (--top->mem_count);
-    bytes_read= read_to_buffer(file, top, key_length);
-    if (bytes_read == (uint32_t) (-1))
-      goto end;
-  }
-  while (bytes_read);
-  res= 0;
-end:
-  return res;
+  assert(elements == 0);
 }
 
 
@@ -578,35 +197,7 @@
 
 bool Unique::walk(tree_walk_action action, void *walk_action_arg)
 {
-  int res;
-  std::vector<unsigned char> merge_buffer;
-
-  if (elements == 0)                       /* the whole tree is in memory */
-    return tree_walk(&tree, action, walk_action_arg, left_root_right);
-
-  /* flush current tree to the file to have some memory for merge buffer */
-  if (flush())
-    return true;
-
-  if (flush_io_cache(file) || reinit_io_cache(file, internal::READ_CACHE, 0L, 0, 0))
-    return true;
-
-  try
-  {
-    merge_buffer.resize(max_in_memory_size);
-  }
-  catch (std::bad_alloc const&)
-  {
-    return true;
-  }
-
-  res= merge_walk(&merge_buffer[0], (ulong) max_in_memory_size, size,
-                  (buffpek_st *) file_ptrs.buffer,
-                  (buffpek_st *) file_ptrs.buffer + file_ptrs.elements,
-                  action, walk_action_arg,
-                  tree.compare, tree.custom_arg, file);
-
-  return res;
+  return tree_walk(&tree, action, walk_action_arg, left_root_right);
 }
 
 /*
@@ -616,78 +207,17 @@
 
 bool Unique::get(Table *table)
 {
-  SORTPARAM sort_param;
-  table->sort.found_records=elements+tree.elements_in_tree;
+  table->sort.found_records= elements+tree.elements_in_tree;
 
-  if (my_b_tell(file) == 0)
+  if ((record_pointers=table->sort.record_pointers= (unsigned char*)
+       malloc(size * tree.elements_in_tree)))
   {
-    /* Whole tree is in memory;  Don't use disk if you don't need to */
-    if ((record_pointers=table->sort.record_pointers= (unsigned char*)
-	 malloc(size * tree.elements_in_tree)))
-    {
-      (void) tree_walk(&tree, (tree_walk_action) unique_write_to_ptrs,
-		       this, left_root_right);
-      return 0;
-    }
+    (void) tree_walk(&tree, (tree_walk_action) unique_write_to_ptrs,
+                     this, left_root_right);
+    return 0;
   }
-  /* Not enough memory; Save the result to file && free memory used by tree */
-  if (flush())
-    return 1;
-
-  internal::IO_CACHE *outfile=table->sort.io_cache;
-  buffpek_st *file_ptr= (buffpek_st*) file_ptrs.buffer;
-  uint32_t maxbuffer= file_ptrs.elements - 1;
-  unsigned char *sort_buffer;
-  internal::my_off_t save_pos;
-  bool error=1;
-
-      /* Open cached file if it isn't open */
-  outfile=table->sort.io_cache= new internal::IO_CACHE;
-
-  if (!outfile || (! my_b_inited(outfile) && open_cached_file(outfile, drizzle_tmpdir.c_str(),TEMP_PREFIX,READ_RECORD_BUFFER, MYF(MY_WME))))
-    return 1;
-  reinit_io_cache(outfile, internal::WRITE_CACHE, 0L, 0, 0);
-
-  memset(&sort_param, 0, sizeof(sort_param));
-  sort_param.max_rows= elements;
-  sort_param.sort_form=table;
-  sort_param.rec_length= sort_param.sort_length= sort_param.ref_length=
-    size;
-  sort_param.keys= (uint32_t) (max_in_memory_size / sort_param.sort_length);
-  sort_param.not_killable=1;
-
-  if (!(sort_buffer=(unsigned char*) malloc((sort_param.keys+1) *
-				            sort_param.sort_length)))
-    return 1;
-  sort_param.unique_buff= sort_buffer+(sort_param.keys*
-				       sort_param.sort_length);
-
-  sort_param.compare= (qsort2_cmp) buffpek_compare;
-  sort_param.cmp_context.key_compare= tree.compare;
-  sort_param.cmp_context.key_compare_arg= tree.custom_arg;
-
-  /* Merge the buffers to one file, removing duplicates */
-  if (merge_many_buff(&sort_param,sort_buffer,file_ptr,&maxbuffer,file))
-    goto err;
-  if (flush_io_cache(file) ||
-      reinit_io_cache(file,internal::READ_CACHE,0L,0,0))
-    goto err;
-  if (merge_buffers(&sort_param, file, outfile, sort_buffer, file_ptr,
-		    file_ptr, file_ptr+maxbuffer,0))
-    goto err;
-  error=0;
-err:
-  if (sort_buffer)
-    free(sort_buffer);
-  if (flush_io_cache(outfile))
-    error=1;
-
-  /* Setup io_cache for reading */
-  save_pos=outfile->pos_in_file;
-  if (reinit_io_cache(outfile,internal::READ_CACHE,0L,0,0))
-    error=1;
-  outfile->end_of_file=save_pos;
-  return error;
+  /* Not enough memory */
+  return 1;
 }
 
 } /* namespace drizzled */

=== modified file 'drizzled/unique.h'
--- drizzled/unique.h	2010-03-30 09:29:02 +0000
+++ drizzled/unique.h	2010-09-09 21:45:53 +0000
@@ -40,13 +40,9 @@
 
 class Unique : public memory::SqlAlloc
 {
-  DYNAMIC_ARRAY file_ptrs;
-  ulong max_elements;
   size_t max_in_memory_size;
-  internal::IO_CACHE *file;
   TREE tree;
   unsigned char *record_pointers;
-  bool flush();
   uint32_t size;
 
 public:
@@ -57,9 +53,7 @@
   ulong elements_in_tree() { return tree.elements_in_tree; }
   inline bool unique_add(void *ptr)
   {
-    if (tree.elements_in_tree > max_elements && flush())
-      return(1);
-    return(!tree_insert(&tree, ptr, 0, tree.custom_arg));
+    return (not tree_insert(&tree, ptr, 0, tree.custom_arg));
   }
 
   bool get(Table *table);

=== modified file 'plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result'
--- plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result	2010-09-08 10:02:37 +0000
+++ plugin/embedded_innodb/test-suite-dir/embedded_innodb/tests/r/type_blob.result	2010-09-10 22:13:46 +0000
@@ -3,10 +3,10 @@
 show columns from t1;
 Field	Type	Null	Default	Default_is_NULL	On_Update
 a	BLOB	YES		YES	
-b	BLOB	YES		YES	
+b	TEXT	YES		YES	
 c	BLOB	YES		YES	
-d	BLOB	YES		YES	
-e	BLOB	YES		YES	
+d	TEXT	YES		YES	
+e	TEXT	YES		YES	
 CREATE TABLE t2 (a varchar(255), b blob, c blob);
 CREATE TABLE t4 (c varchar(16383) not null);
 show columns from t2;

=== modified file 'plugin/heap/ha_heap.cc'
--- plugin/heap/ha_heap.cc	2010-09-08 23:07:51 +0000
+++ plugin/heap/ha_heap.cc	2010-09-09 18:27:05 +0000
@@ -246,7 +246,7 @@
                              table->getShare()->getTableName(),
                              table->getShare()->getPath());
 
-  if (new_handler && !new_handler->ha_open(identifier, table, file->getShare()->name.c_str(), table->db_stat,
+  if (new_handler && !new_handler->ha_open(identifier, table, table->db_stat,
                                            HA_OPEN_IGNORE_IF_LOCKED))
     return new_handler;
   return NULL;

=== modified file 'plugin/heap/hp_dspace.cc'
--- plugin/heap/hp_dspace.cc	2010-09-08 23:46:54 +0000
+++ plugin/heap/hp_dspace.cc	2010-09-09 18:27:05 +0000
@@ -241,7 +241,7 @@
 
   info->chunk_count++;
   curr_chunk= ((unsigned char*) info->block.level_info[0].last_blocks +
-    block_pos * info->block.recbuffer);
+               block_pos * info->block.recbuffer);
 
 
   return curr_chunk;

